@@ -63,9 +63,6 @@ public:
     /// reimplemented
     void keyPressEvent(QKeyEvent *event) override;
 
-    /// For behavior as selection tool and with initial shift-key
-    virtual bool listeningToModifiers();
-
     /**
     * Returns true if path has been started
     */
@@ -73,6 +70,8 @@ public:
 
     bool tryMergeInPathShape(KoPathShape *pathShape);
 
+    void setEnableClosePathShortcut(bool value);
+
 public Q_SLOTS:
     /// reimplemented
     void activate(ToolActivation activation, const QSet<KoShape*> &shapes) override;
@@ -81,6 +80,9 @@ public Q_SLOTS:
     /// reimplemented
     void documentResourceChanged(int key, const QVariant & res) override;
 
+Q_SIGNALS:
+    void sigUpdateAutoSmoothCurvesGUI(bool value);
+
 protected:
     /**
      * Add path shape to document.
@@ -109,5 +111,6 @@ private:
     Q_DECLARE_PRIVATE(KoCreatePathTool)
     Q_PRIVATE_SLOT(d_func(), void angleDeltaChanged(int))
     Q_PRIVATE_SLOT(d_func(), void angleSnapChanged(int))
+    Q_PRIVATE_SLOT(d_func(), void autoSmoothCurvesChanged(bool))
 };
 #endif
