@@ -83,7 +83,7 @@ struct PathConnectionPoint {
             reset();
             return;
         }
-        // check if path is still part of the docment
+        // check if path is still part of the document
         if (!canvas->shapeManager()->shapes().contains(path)) {
             reset();
             return;
@@ -420,12 +420,12 @@ public:
     void autoSmoothCurvesChanged(bool value) {
         autoSmoothCurves = value;
 
-        KisConfig cfg;
+        KisConfig cfg(false);
         cfg.setAutoSmoothBezierCurves(value);
     }
 
     void loadAutoSmoothValueFromConfig() {
-        KisConfig cfg;
+        KisConfig cfg(true);
         autoSmoothCurves = cfg.autoSmoothBezierCurves();
 
         emit q->sigUpdateAutoSmoothCurvesGUI(autoSmoothCurves);
