@@ -176,7 +176,7 @@ bool KoConnectionShapePrivate::intersects(const QPointF &p1, const QPointF &d1,
     // use cross product to check if rays intersects at all
     qreal cp = crossProd(d1, d2);
     if (cp == 0.0) {
-        // rays are parallel or coincidient
+        // rays are parallel or coincident
         if (p1.x() == p2.x() && d1.x() == 0.0 && d1.y() != d2.y()) {
             // vertical, coincident
             isect = 0.5 * (p1 + p2);
@@ -374,16 +374,16 @@ void KoConnectionShape::saveOdf(KoShapeSavingContext & context) const
         context.xmlWriter().addAttribute("draw:start-glue-point", d->connectionPointId1);
     } else {
         QPointF p(shapeToDocument(d->handles[StartHandle]) * context.shapeOffset(this));
-        context.xmlWriter().addAttributePt("svg:x1", p.x());
-        context.xmlWriter().addAttributePt("svg:y1", p.y());
+        context.xmlWriter().addAttribute("svg:x1", p.x());
+        context.xmlWriter().addAttribute("svg:y1", p.y());
     }
     if (d->shape2) {
         context.xmlWriter().addAttribute("draw:end-shape", context.xmlid(d->shape2, "shape", KoElementReference::Counter).toString());
         context.xmlWriter().addAttribute("draw:end-glue-point", d->connectionPointId2);
     } else {
         QPointF p(shapeToDocument(d->handles[EndHandle]) * context.shapeOffset(this));
-        context.xmlWriter().addAttributePt("svg:x2", p.x());
-        context.xmlWriter().addAttributePt("svg:y2", p.y());
+        context.xmlWriter().addAttribute("svg:x2", p.x());
+        context.xmlWriter().addAttribute("svg:y2", p.y());
     }
 
     // write the path data
