@@ -34,7 +34,6 @@
 #include "kis_selection.h"
 #include "kis_node_commands_adapter.h"
 #include "kis_group_layer.h"
-#include "kis_statusbar.h"
 #include "kis_progress_widget.h"
 #include "kis_config.h"
 #include "KisPart.h"
@@ -102,7 +101,7 @@ void KisImportCatcher::slotLoadingFinished()
     KisImageWSP importedImage = m_d->doc->image();
     importedImage->waitForDone();
 
-    if (importedImage && importedImage->projection()->exactBounds().isValid()) {
+    if (importedImage && importedImage->bounds().isValid()) {
         if (m_d->layerType == "KisPaintLayer") {
             KisPaintDeviceSP dev = importedImage->projection();
             adaptClipToImageColorSpace(dev, m_d->view->image());
@@ -141,12 +140,10 @@ KisImportCatcher::~KisImportCatcher()
 
 void KisImportCatcher::adaptClipToImageColorSpace(KisPaintDeviceSP dev, KisImageSP image)
 {
-    KisConfig cfg;
-    qDebug() << "dev" << dev->colorSpace() << "image" << image->colorSpace() << "cfg" << cfg.convertToImageColorspaceOnImport();
+    KisConfig cfg(true);
     if (cfg.convertToImageColorspaceOnImport() && *dev->colorSpace() != *image->colorSpace()) {
         /// XXX: do we need intent here?
-        KUndo2Command* cmd = dev->convertTo(image->colorSpace());
-        delete cmd;
+        dev->convertTo(image->colorSpace());
     }
 }
 
