@@ -49,6 +49,9 @@
 #include <KoSelectedShapesProxy.h>
 #include <KoSelection.h>
 #include <commands/KoKeepShapesSelectedCommand.h>
+#include "kis_selection_mask.h"
+#include "kis_shape_selection.h"
+#include "kis_processing_applicator.h"
 
 
 KisToolShape::KisToolShape(KoCanvasBase * canvas, const QCursor & cursor)
@@ -137,6 +140,29 @@ qreal KisToolShape::currentStrokeWidth() const
     return canvas()->unit().fromUserValue(sizeInPx);
 }
 
+KisToolShape::ShapeAddInfo KisToolShape::shouldAddShape(KisNodeSP currentNode) const
+{
+    ShapeAddInfo info;
+
+    if (currentNode->inherits("KisShapeLayer")) {
+        info.shouldAddShape = true;
+    } else if (KisSelectionMask *mask = dynamic_cast<KisSelectionMask*>(currentNode.data())) {
+        if (mask->selection()->hasShapeSelection()) {
+            info.shouldAddShape = true;
+            info.shouldAddSelectionShape = true;
+        }
+    }
+
+    return info;
+}
+
+void KisToolShape::ShapeAddInfo::markAsSelectionShapeIfNeeded(KoShape *shape) const
+{
+    if (this->shouldAddSelectionShape) {
+        shape->setUserData(new KisShapeSelectionMarker());
+    }
+}
+
 void KisToolShape::addShape(KoShape* shape)
 {
     KoImageCollection* imageCollection = canvas()->shapeController()->resourceManager()->imageCollection();
@@ -198,13 +224,13 @@ void KisToolShape::addShape(KoShape* shape)
     parentCommand->setText(cmd->text());
     new KoKeepShapesSelectedCommand(oldSelectedShapes, {shape}, canvas()->selectedShapesProxy(), true, parentCommand);
 
-    canvas()->addCommand(parentCommand);
+    KisProcessingApplicator::runSingleCommandStroke(image(), cmd);
 }
 
 void KisToolShape::addPathShape(KoPathShape* pathShape, const KUndo2MagicString& name)
 {
     KisNodeSP node = currentNode();
-    if (!node || !blockUntilOperationsFinished()) {
+    if (!node) {
         return;
     }
 
@@ -228,7 +254,4 @@ void KisToolShape::addPathShape(KoPathShape* pathShape, const KUndo2MagicString&
         addShape(pathShape);
 
     }
-
-    notifyModified();
 }
-
