@@ -66,8 +66,8 @@ class KRITAUI_EXPORT KisMultiDoubleFilterWidget : public KisConfigWidget
     Q_OBJECT
 public:
     KisMultiDoubleFilterWidget(const QString & filterid, QWidget * parent, const QString & caption, vKisDoubleWidgetParam dwparam);
-    virtual void setConfiguration(const KisPropertiesConfigurationSP  cfg);
-    virtual KisPropertiesConfigurationSP configuration() const;
+    void setConfiguration(const KisPropertiesConfigurationSP  cfg) override;
+    KisPropertiesConfigurationSP configuration() const override;
 public:
     inline qint32 nbValues() const {
         return m_nbdoubleWidgets;
