@@ -5,7 +5,8 @@
  *
  *  This library is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
- *  the Free Software Foundation; version 2.1 of the License.
+ *  the Free Software Foundation; version 2 of the License, or
+ *  (at your option) any later version.
  *
  *  This library is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -48,6 +49,8 @@
 #include <kis_painting_assistants_decoration.h>
 #include "kis_global.h"
 #include "VanishingPointAssistant.h"
+#include "EditAssistantsCommand.h"
+#include <kis_undo_adapter.h>
 
 #include <math.h>
 
@@ -98,6 +101,8 @@ void KisAssistantTool::deactivate()
 void KisAssistantTool::beginPrimaryAction(KoPointerEvent *event)
 {
     setMode(KisTool::PAINT_MODE);
+    m_origAssistantList = cloneAssistantList(m_canvas->paintingAssistantsDecoration()->assistants());
+
     bool newAssistantAllowed = true;
 
     KisPaintingAssistantsDecorationSP canvasDecoration = m_canvas->paintingAssistantsDecoration();
@@ -270,13 +275,13 @@ void KisAssistantTool::beginPrimaryAction(KoPointerEvent *event)
             } else if (m_handleDrag == assistant->handles()[1]) {
                 m_dragStart = *assistant->handles()[0];
             } else if (m_handleDrag == assistant->handles()[2]) {
-                m_dragStart = assistant->buttonPosition();
+                m_dragStart = assistant->getEditorPosition();
                 m_radius = QLineF(m_dragStart, *assistant->handles()[0]);
                 m_snapIsRadial = true;
             }
 
         } else {
-            m_dragStart = assistant->buttonPosition();
+            m_dragStart = assistant->getEditorPosition();
             m_snapIsRadial = false;
         }
     }
@@ -295,22 +300,21 @@ void KisAssistantTool::beginPrimaryAction(KoPointerEvent *event)
     m_assistantDrag.clear();
     Q_FOREACH (KisPaintingAssistantSP assistant, m_canvas->paintingAssistantsDecoration()->assistants()) {
 
-        // This code contains the click event behavior.
-        QTransform initialTransform = m_canvas->coordinatesConverter()->documentToWidgetTransform();
-        QPointF actionsPosition = initialTransform.map(assistant->buttonPosition());
+        AssistantEditorData editorShared; // shared position data between assistant tool and decoration
+        const KisCoordinatesConverter *converter = m_canvas->coordinatesConverter();
 
+        // This code contains the click event behavior.
+        QTransform initialTransform = converter->documentToWidgetTransform();
+        QPointF actionsPosition = initialTransform.map(assistant->viewportConstrainedEditorPosition(converter, editorShared.boundingSize));
 
         // for UI editor widget controls with move, show, and delete -- disregard document transforms like rotating and mirroring.
         // otherwise the UI controls get awkward to use when they are at 45 degree angles or the order of controls gets flipped backwards
-        QPointF uiMousePosition = initialTransform.map( canvasDecoration->snapToGuide(event, QPointF(), false));
-
-        AssistantEditorData editorShared; // shared position data between assistant tool and decoration
+        QPointF uiMousePosition = initialTransform.map(canvasDecoration->snapToGuide(event, QPointF(), false));
 
         QPointF iconMovePosition(actionsPosition + editorShared.moveIconPosition);
         QPointF iconSnapPosition(actionsPosition + editorShared.snapIconPosition);
         QPointF iconDeletePosition(actionsPosition + editorShared.deleteIconPosition);
 
-
         QRectF deleteRect(iconDeletePosition, QSizeF(editorShared.deleteIconSize, editorShared.deleteIconSize));
         QRectF visibleRect(iconSnapPosition, QSizeF(editorShared.snapIconSize, editorShared.snapIconSize));
         QRectF moveRect(iconMovePosition, QSizeF(editorShared.moveIconSize, editorShared.moveIconSize));
@@ -345,7 +349,7 @@ void KisAssistantTool::beginPrimaryAction(KoPointerEvent *event)
             assistantSelected(assistant); // whatever handle is the closest contains the selected assistant
         }
     }
-    if (newAssistantAllowed==true){//don't make a new assistant when I'm just toogling visiblity//
+    if (newAssistantAllowed==true){//don't make a new assistant when I'm just toogling visibility//
         QString key = m_options.availableAssistantsComboBox->model()->index( m_options.availableAssistantsComboBox->currentIndex(), 0 ).data(Qt::UserRole).toString();
         m_newAssistant = toQShared(KisPaintingAssistantFactoryRegistry::instance()->get(key)->createPaintingAssistant());
         m_internalMode = MODE_CREATION;
@@ -508,16 +512,21 @@ void KisAssistantTool::endPrimaryAction(KoPointerEvent *event)
 {
     setMode(KisTool::HOVER_MODE);
 
-    if (m_handleDrag) {
-        if (!(event->modifiers() & Qt::ShiftModifier) && m_handleCombine) {
-            m_handleCombine->mergeWith(m_handleDrag);
-            m_handleCombine->uncache();
-            m_handles = m_canvas->paintingAssistantsDecoration()->handles();
+    if (m_handleDrag || m_assistantDrag) {
+        if (m_handleDrag) {
+            if (!(event->modifiers() & Qt::ShiftModifier) && m_handleCombine) {
+                m_handleCombine->mergeWith(m_handleDrag);
+                m_handleCombine->uncache();
+                m_handles = m_canvas->paintingAssistantsDecoration()->handles();
+            }
+            m_handleDrag = m_handleCombine = 0;
+        } else {
+            m_assistantDrag.clear();
         }
-        m_handleDrag = m_handleCombine = 0;
-
-    } else if (m_assistantDrag) {
-        m_assistantDrag.clear();
+        dbgUI << "creating undo command...";
+        KUndo2Command *command = new EditAssistantsCommand(m_canvas, m_origAssistantList, cloneAssistantList(m_canvas->paintingAssistantsDecoration()->assistants()));
+        m_canvas->viewManager()->undoAdapter()->addCommand(command);
+        dbgUI << "done";
     } else if(m_internalMode == MODE_DRAGGING_TRANSLATING_TWONODES) {
         addAssistant();
         m_internalMode = MODE_CREATION;
@@ -529,29 +538,52 @@ void KisAssistantTool::endPrimaryAction(KoPointerEvent *event)
     m_canvas->updateCanvas(); // TODO update only the relevant part of the canvas
 }
 
+QList<KisPaintingAssistantSP> KisAssistantTool::cloneAssistantList(const QList<KisPaintingAssistantSP> &list) const
+{
+    QMap<KisPaintingAssistantHandleSP, KisPaintingAssistantHandleSP> handleMap;
+    QList<KisPaintingAssistantSP> clonedList;
+    dbgUI << "cloning assistants...";
+    for (auto i = list.begin(); i != list.end(); ++i) {
+        clonedList << (*i)->clone(handleMap);
+    }
+    dbgUI << "done";
+    return clonedList;
+}
+
 void KisAssistantTool::addAssistant()
 {
     m_canvas->paintingAssistantsDecoration()->addAssistant(m_newAssistant);
+
+    KisAbstractPerspectiveGrid* grid = dynamic_cast<KisAbstractPerspectiveGrid*>(m_newAssistant.data());
+    if (grid) {
+        m_canvas->viewManager()->canvasResourceProvider()->addPerspectiveGrid(grid);
+    }
+
+    QList<KisPaintingAssistantSP> assistants = m_canvas->paintingAssistantsDecoration()->assistants();
+    KUndo2Command *addAssistantCmd = new EditAssistantsCommand(m_canvas, m_origAssistantList, cloneAssistantList(assistants), EditAssistantsCommand::ADD, assistants.indexOf(m_newAssistant));
+    m_canvas->viewManager()->undoAdapter()->addCommand(addAssistantCmd);
+
     m_handles = m_canvas->paintingAssistantsDecoration()->handles();
     m_canvas->paintingAssistantsDecoration()->setSelectedAssistant(m_newAssistant);
     updateToolOptionsUI(); // vanishing point assistant will get an extra option
 
-    KisAbstractPerspectiveGrid* grid = dynamic_cast<KisAbstractPerspectiveGrid*>(m_newAssistant.data());
-    if (grid) {
-        m_canvas->viewManager()->resourceProvider()->addPerspectiveGrid(grid);
-    }
     m_newAssistant.clear();
 }
 
 void KisAssistantTool::removeAssistant(KisPaintingAssistantSP assistant)
 {
+    QList<KisPaintingAssistantSP> assistants = m_canvas->paintingAssistantsDecoration()->assistants();
+
     KisAbstractPerspectiveGrid* grid = dynamic_cast<KisAbstractPerspectiveGrid*>(assistant.data());
     if (grid) {
-        m_canvas->viewManager()->resourceProvider()->removePerspectiveGrid(grid);
+        m_canvas->viewManager()->canvasResourceProvider()->removePerspectiveGrid(grid);
     }
     m_canvas->paintingAssistantsDecoration()->removeAssistant(assistant);
-    m_handles = m_canvas->paintingAssistantsDecoration()->handles();
 
+    KUndo2Command *removeAssistantCmd = new EditAssistantsCommand(m_canvas, m_origAssistantList, cloneAssistantList(m_canvas->paintingAssistantsDecoration()->assistants()), EditAssistantsCommand::REMOVE, assistants.indexOf(assistant));
+    m_canvas->viewManager()->undoAdapter()->addCommand(removeAssistantCmd);
+
+    m_handles = m_canvas->paintingAssistantsDecoration()->handles();
     m_canvas->paintingAssistantsDecoration()->deselectAssistant();
     updateToolOptionsUI();
 }
@@ -688,7 +720,7 @@ void KisAssistantTool::paint(QPainter& _gc, const KoViewConverter &_converter)
 
 void KisAssistantTool::removeAllAssistants()
 {
-    m_canvas->viewManager()->resourceProvider()->clearPerspectiveGrids();
+    m_canvas->viewManager()->canvasResourceProvider()->clearPerspectiveGrids();
     m_canvas->paintingAssistantsDecoration()->removeAll();
     m_handles = m_canvas->paintingAssistantsDecoration()->handles();
     m_canvas->updateCanvas();
@@ -799,7 +831,7 @@ void KisAssistantTool::loadAssistants()
                         m_canvas->paintingAssistantsDecoration()->addAssistant(assistant);
                         KisAbstractPerspectiveGrid* grid = dynamic_cast<KisAbstractPerspectiveGrid*>(assistant.data());
                         if (grid) {
-                            m_canvas->viewManager()->resourceProvider()->addPerspectiveGrid(grid);
+                            m_canvas->viewManager()->canvasResourceProvider()->addPerspectiveGrid(grid);
                         }
                     } else {
                         errors = true;
@@ -926,13 +958,13 @@ QWidget *KisAssistantTool::createOptionWidget()
         connect(m_options.loadAssistantButton, SIGNAL(clicked()), SLOT(loadAssistants()));
         connect(m_options.deleteAllAssistantsButton, SIGNAL(clicked()), SLOT(removeAllAssistants()));
 
-        connect(m_options.assistantsColor, SIGNAL(changed(const QColor&)), SLOT(slotGlobalAssistantsColorChanged(const QColor&)));
+        connect(m_options.assistantsColor, SIGNAL(changed(QColor)), SLOT(slotGlobalAssistantsColorChanged(QColor)));
         connect(m_options.assistantsGlobalOpacitySlider, SIGNAL(valueChanged(int)), SLOT(slotGlobalAssistantOpacityChanged()));
 
 
         connect(m_options.vanishingPointAngleSpinbox, SIGNAL(valueChanged(double)), this, SLOT(slotChangeVanishingPointAngle(double)));
 
-        ENTER_FUNCTION() << ppVar(m_canvas) << ppVar(m_canvas && m_canvas->paintingAssistantsDecoration());
+        //ENTER_FUNCTION() << ppVar(m_canvas) << ppVar(m_canvas && m_canvas->paintingAssistantsDecoration());
 
         // initialize UI elements with existing data if possible
         if (m_canvas && m_canvas->paintingAssistantsDecoration()) {
@@ -941,7 +973,7 @@ QWidget *KisAssistantTool::createOptionWidget()
             QColor opaqueColor = color;
             opaqueColor.setAlpha(255);
 
-            ENTER_FUNCTION() << ppVar(opaqueColor);
+            //ENTER_FUNCTION() << ppVar(opaqueColor);
 
             m_options.assistantsColor->setColor(opaqueColor);
             m_options.customAssistantColorButton->setColor(opaqueColor);
