@@ -29,6 +29,7 @@
 #include <QDropEvent>
 #include <QMenu>
 #include <QScrollBar>
+#include <QScroller>
 #include <QDrag>
 #include <QInputDialog>
 #include <QClipboard>
@@ -36,6 +37,7 @@
 #include "config-qtmultimedia.h"
 
 #include "KSharedConfig"
+#include "KisKineticScroller.h"
 
 #include "kis_zoom_button.h"
 #include "kis_icon_utils.h"
@@ -45,6 +47,7 @@
 #include "kis_signal_compressor.h"
 #include "kis_time_range.h"
 #include "kis_color_label_selector_widget.h"
+#include "kis_keyframe_channel.h"
 #include "kis_slider_spin_box.h"
 #include <KisImportExportManager.h>
 #include <kis_signals_blocker.h>
@@ -193,7 +196,7 @@ TimelineFramesView::TimelineFramesView(QWidget *parent)
     connect(m_d->existingLayersMenu, SIGNAL(aboutToShow()), SLOT(slotUpdateLayersMenu()));
     connect(m_d->existingLayersMenu, SIGNAL(triggered(QAction*)), SLOT(slotAddExistingLayer(QAction*)));
 
-    connect(m_d->layersHeader, SIGNAL(sigRequestContextMenu(const QPoint&)), SLOT(slotLayerContextMenuRequested(const QPoint&)));
+    connect(m_d->layersHeader, SIGNAL(sigRequestContextMenu(QPoint)), SLOT(slotLayerContextMenuRequested(QPoint)));
 
     m_d->addLayersButton->setMenu(m_d->layerEditingMenu);
 
@@ -271,6 +274,14 @@ TimelineFramesView::TimelineFramesView(QWidget *parent)
     setFramesPerSecond(12);
     setHorizontalScrollMode(QAbstractItemView::ScrollPerPixel);
 
+    {
+        QScroller *scroller = KisKineticScroller::createPreconfiguredScroller(this);
+        if( scroller ) {
+            connect(scroller, SIGNAL(stateChanged(QScroller::State)),
+                    this, SLOT(slotScrollerStateChanged(QScroller::State)));
+        }
+    }
+
     connect(&m_d->selectionChangedCompressor, SIGNAL(timeout()),
             SLOT(slotSelectionChanged()));
     connect(&m_d->selectionChangedCompressor, SIGNAL(timeout()),
@@ -395,13 +406,13 @@ void TimelineFramesView::setModel(QAbstractItemModel *model)
 
     QTableView::setModel(model);
 
-    connect(m_d->model, SIGNAL(headerDataChanged(Qt::Orientation, int, int)),
-            this, SLOT(slotHeaderDataChanged(Qt::Orientation, int, int)));
+    connect(m_d->model, SIGNAL(headerDataChanged(Qt::Orientation,int,int)),
+            this, SLOT(slotHeaderDataChanged(Qt::Orientation,int,int)));
 
     connect(m_d->model, SIGNAL(dataChanged(QModelIndex,QModelIndex)),
             this, SLOT(slotDataChanged(QModelIndex,QModelIndex)));
 
-    connect(m_d->model, SIGNAL(rowsRemoved(const QModelIndex&, int, int)),
+    connect(m_d->model, SIGNAL(rowsRemoved(QModelIndex,int,int)),
             this, SLOT(slotReselectCurrentIndex()));
 
     connect(m_d->model, SIGNAL(sigInfiniteTimelineUpdateNeeded()),
@@ -410,7 +421,7 @@ void TimelineFramesView::setModel(QAbstractItemModel *model)
     connect(m_d->model, SIGNAL(sigAudioChannelChanged()),
             this, SLOT(slotUpdateAudioActions()));
 
-    connect(selectionModel(), SIGNAL(selectionChanged(const QItemSelection &, const QItemSelection &)),
+    connect(selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)),
             &m_d->selectionChangedCompressor, SLOT(start()));
 
     connect(m_d->model, SIGNAL(sigEnsureRowVisible(int)), SLOT(slotEnsureRowVisible(int)));
@@ -457,9 +468,7 @@ void TimelineFramesView::slotColorLabelChanged(int label)
     Q_FOREACH(QModelIndex index, selectedIndexes()) {
         m_d->model->setData(index, label, TimelineFramesModel::FrameColorLabelIndexRole);
     }
-
-    KisImageConfig config;
-    config.setDefaultFrameColorLabel(label);
+    KisImageConfig(false).setDefaultFrameColorLabel(label);
 }
 
 void TimelineFramesView::slotSelectAudioChannelFile()
@@ -555,6 +564,10 @@ void TimelineFramesView::slotUpdateInfiniteFramesCount()
     m_d->model->setLastVisibleFrame(calculatedIndex);
 }
 
+void TimelineFramesView::slotScrollerStateChanged( QScroller::State state ) {
+    KisKineticScroller::updateCursor(this, state);
+}
+
 void TimelineFramesView::currentChanged(const QModelIndex &current, const QModelIndex &previous)
 {
     QTableView::currentChanged(current, previous);
@@ -899,6 +912,9 @@ void TimelineFramesView::dropEvent(QDropEvent *event)
     m_d->dragInProgress = false;
     m_d->model->setScrubState(false);
 
+    if (event->keyboardModifiers() & Qt::ControlModifier) {
+        event->setDropAction(Qt::CopyAction);
+    }
     QAbstractItemView::dropEvent(event);
     m_d->dragWasSuccessful = event->isAccepted();
 }
@@ -1020,8 +1036,7 @@ void TimelineFramesView::mousePressEvent(QMouseEvent *event)
             } else {
                 {
                     KisSignalsBlocker b(m_d->colorSelector);
-                    KisImageConfig cfg;
-                    const int labelIndex = cfg.defaultFrameColorLabel();
+                    const int labelIndex = KisImageConfig(true).defaultFrameColorLabel();
                     m_d->colorSelector->setCurrentIndex(labelIndex);
                 }
 
@@ -1405,7 +1420,32 @@ void TimelineFramesView::insertOrRemoveHoldFrames(int count, bool entireColumn)
     }
 
     if (!indexes.isEmpty()) {
+
+        // add extra columns to the end of the timeline if we are adding hold frames
+        // they will be truncated if we don't do this
+        if (count > 0) {
+            // Scan all the layers and find out what layer has the most keyframes
+            // only keep a reference of layer that has the most keyframes
+            int keyframesInLayerNode = 0;
+            Q_FOREACH (const QModelIndex &index, indexes) {
+                KisNodeSP layerNode = m_d->model->nodeAt(index);
+
+                KisKeyframeChannel *channel = layerNode->getKeyframeChannel(KisKeyframeChannel::Content.id());
+                if (!channel) continue;
+
+                if (keyframesInLayerNode < channel->allKeyframeIds().count()) {
+                   keyframesInLayerNode = channel->allKeyframeIds().count();
+                }
+            }
+            m_d->model->setLastVisibleFrame(m_d->model->columnCount() + count*keyframesInLayerNode);
+        }
+
+
         m_d->model->insertHoldFrames(indexes, count);
+
+        // bulk adding frames can add too many
+        // trim timeline to clean up extra frames that might have been added
+        slotUpdateInfiniteFramesCount();
     }
 }
 
