@@ -43,13 +43,16 @@
 #include <KisDocument.h>
 
 #include "kis_node_manager.h"
+#include "kis_selection_manager.h"
 #include "kis_signals_blocker.h"
 #include <boost/operators.hpp>
+#include "KisMoveBoundsCalculationJob.h"
+
 
 struct KisToolMoveState : KisToolChangesTrackerData, boost::equality_comparable<KisToolMoveState>
 {
     KisToolMoveState(QPoint _accumulatedOffset) : accumulatedOffset(_accumulatedOffset) {}
-    KisToolChangesTrackerData* clone() const { return new KisToolMoveState(*this); }
+    KisToolChangesTrackerData* clone() const override { return new KisToolMoveState(*this); }
 
     bool operator ==(const KisToolMoveState &rhs) {
         return accumulatedOffset == rhs.accumulatedOffset;
@@ -90,6 +93,7 @@ KisToolMove::KisToolMove(KoCanvasBase *canvas)
     connect(this, SIGNAL(moveInNewPosition(QPoint)), m_optionsWidget, SLOT(slotSetTranslate(QPoint)), Qt::UniqueConnection);
 
     connect(qobject_cast<KisCanvas2*>(canvas)->viewManager()->nodeManager(), SIGNAL(sigUiNeedChangeSelectedNodes(KisNodeList)), this, SLOT(slotNodeChanged(KisNodeList)), Qt::UniqueConnection);
+    connect(qobject_cast<KisCanvas2*>(canvas)->viewManager()->selectionManager(), SIGNAL(currentSelectionChanged()), this, SLOT(slotSelectionChanged()), Qt::UniqueConnection);
 }
 
 KisToolMove::~KisToolMove()
@@ -165,21 +169,6 @@ bool KisToolMove::startStrokeImpl(MoveToolMode mode, const QPoint *pos)
         return true;
     }
 
-    /**
-     * FIXME: The move tool is not completely asynchronous, it
-     * needs the content of the layer and/or selection to calculate
-     * the bounding rectange. Technically, we can move its calculation
-     * into the stroke itself and pass it back to the tool via a signal.
-     *
-     * But currently, we will just disable starting a new stroke
-     * asynchronously.
-     */
-    if (!blockUntilOperationsFinished()) {
-        return false;
-    }
-
-    initHandles(nodes);
-
     KisStrokeStrategy *strategy;
 
     KisPaintLayerSP paintLayer = node ?
@@ -189,16 +178,32 @@ bool KisToolMove::startStrokeImpl(MoveToolMode mode, const QPoint *pos)
         (!selection->selectedRect().isEmpty() &&
          !selection->selectedExactRect().isEmpty())) {
 
-        strategy =
+        MoveSelectionStrokeStrategy *moveStrategy =
             new MoveSelectionStrokeStrategy(paintLayer,
                                             selection,
                                             image.data(),
                                             image.data());
+
+        connect(moveStrategy,
+                SIGNAL(sigHandlesRectCalculated(const QRect&)),
+                SLOT(slotHandlesRectCalculated(const QRect&)));
+
+        strategy = moveStrategy;
+
     } else {
-        strategy =
+
+        MoveStrokeStrategy *moveStrategy =
             new MoveStrokeStrategy(nodes, image.data(), image.data());
+        connect(moveStrategy,
+                SIGNAL(sigHandlesRectCalculated(const QRect&)),
+                SLOT(slotHandlesRectCalculated(const QRect&)));
+
+        strategy = moveStrategy;
     }
 
+    // disable outline feedback until the stroke calcualtes
+    // correct bounding rect
+    m_handlesRect = QRect();
     m_strokeId = image->startStroke(strategy);
     m_currentlyProcessingNodes = nodes;
     m_accumulatedOffset = QPoint();
@@ -219,6 +224,7 @@ QPoint KisToolMove::currentOffset() const
 void KisToolMove::notifyGuiAfterMove(bool showFloatingMessage)
 {
     if (!m_optionsWidget) return;
+    if (m_handlesRect.isEmpty()) return;
 
     const QPoint currentTopLeft = m_handlesRect.topLeft() + currentOffset();
 
@@ -240,7 +246,7 @@ void KisToolMove::notifyGuiAfterMove(bool showFloatingMessage)
     }
 }
 
-bool KisToolMove::tryEndPreviousStroke(KisNodeList nodes)
+bool KisToolMove::tryEndPreviousStroke(const KisNodeList &nodes)
 {
     if (!m_strokeId) return false;
 
@@ -265,6 +271,12 @@ void KisToolMove::commitChanges()
     m_changesTracker.commitConfig(newState);
 }
 
+void KisToolMove::slotHandlesRectCalculated(const QRect &handlesRect)
+{
+    m_handlesRect = handlesRect;
+    notifyGuiAfterMove(false);
+}
+
 void KisToolMove::moveDiscrete(MoveDirection direction, bool big)
 {
     if (mode() == KisTool::PAINT_MODE) return;  // Don't interact with dragging
@@ -333,7 +345,7 @@ void KisToolMove::paint(QPainter& gc, const KoViewConverter &converter)
 {
     Q_UNUSED(converter);
 
-    if (m_strokeId) {
+    if (m_strokeId && !m_handlesRect.isEmpty()) {
         QPainterPath handles;
         handles.addRect(m_handlesRect.translated(currentOffset()));
 
@@ -342,24 +354,6 @@ void KisToolMove::paint(QPainter& gc, const KoViewConverter &converter)
     }
 }
 
-void KisToolMove::initHandles(const KisNodeList &nodes)
-{
-    /**
-     * The handles should be initialized only once, **before** the start of
-     * the stroke. If the nodes change, we should restart the stroke.
-     */
-    KIS_SAFE_ASSERT_RECOVER_NOOP(!m_strokeId);
-
-    m_handlesRect = QRect();
-    for (KisNodeSP node : nodes) {
-        node->exactBounds();
-        m_handlesRect |= node->exactBounds();
-    }
-    if (image()->globalSelection()) {
-        m_handlesRect &= image()->globalSelection()->selectedExactRect();
-    }
-}
-
 void KisToolMove::deactivate()
 {
     m_actionConnections.clear();
@@ -653,14 +647,36 @@ void KisToolMove::moveBySpinY(int newY)
     setMode(KisTool::HOVER_MODE);
 }
 
-void KisToolMove::slotNodeChanged(KisNodeList nodes)
+void KisToolMove::requestHandlesRectUpdate()
+{
+    KisResourcesSnapshotSP resources =
+        new KisResourcesSnapshot(image(), currentNode(), canvas()->resourceManager());
+    KisSelectionSP selection = resources->activeSelection();
+
+    KisMoveBoundsCalculationJob *job = new KisMoveBoundsCalculationJob(this->selectedNodes(),
+                                                                       selection, this);
+    connect(job,
+            SIGNAL(sigCalcualtionFinished(const QRect&)),
+            SLOT(slotHandlesRectCalculated(const QRect &)));
+
+    KisImageSP image = this->image();
+    image->addSpontaneousJob(job);
+
+    notifyGuiAfterMove(false);
+}
+
+void KisToolMove::slotNodeChanged(const KisNodeList &nodes)
 {
     if (m_strokeId && !tryEndPreviousStroke(nodes)) {
         return;
     }
+    requestHandlesRectUpdate();
+}
 
-    initHandles(nodes);
-    notifyGuiAfterMove(false);
+void KisToolMove::slotSelectionChanged()
+{
+    if (m_strokeId) return;
+    requestHandlesRectUpdate();
 }
 
 QList<QAction *> KisToolMoveFactory::createActionsImpl()
